/*
 *  Config file handling for Watt-32 port of tcptrace
 *
 *  G. Vanem <giva@bgnett.no>  1999
 */

#include "tcptrace.h"

#if !defined(MSDOS) && !defined(__DJGPP__)
#error This file is for DOS+djgpp only.
#endif

#include <limits.h>
#include <dos.h>
#include <tcp.h>   /* Watt-32 API */

/*
 *  These are from wattcp
 */
extern void ReadHostFile (const char *fname);
extern void ReadServFile (const char *fname);

extern char _hostname [51];
extern char  defaultdomain [51];

/*
 * Substitute all publics i wattcp's pcconfig.c
 */
WORD   sock_inactive = 20;
WORD   sock_data_timeout = 120;
void  (*usr_init)()  = NULL;
char   *watfname     = "WATTCP.CFG";
WORD   sock_delay    = 20;
WORD   multihomes    = 0;
int   _last_cookie   = 0;
int   _ctrace_on     = 0;
DWORD _cookie [MAX_COOKIES];

void _add_server (void) {}


/*
 * Convert hexstring "0x??" to hex
 */
BYTE atox (char *value)
{
  int hi = toupper (value[2]);
  int lo = toupper (value[3]);

  hi -= isdigit (hi) ? '0' : 'A'-10;
  lo -= isdigit (lo) ? '0' : 'A'-10;
  return ((hi << 4) + lo);
}

/*
 * Return a string with a environment variable expanded (only 1).
 *
 * E.g. if environment variable ETC is "c:\network\watt\bin",
 * "$(ETC)\hosts" becomes "c:\network\watt\bin\hosts"
 */
char * ExpandVarStr (const char *str)
{
  static char buf[PATH_MAX];
  char   env[30];
  char  *e, *p  = strstr (str,"$(");
  char  *dollar = p;
  int    i;

  if (!p)
     return (char*) str;

  for (i = 0, p += 2; i < sizeof(env)-1; i++)
  {
    if (*p == ')')
       break;
    env[i] = *p++;
  }

  if (*p++ != ')')
     return (char*) str;

  env[i] = 0;
  strupr (env);
  if ((e = getenv(env)) == NULL)
  {
    e = env;
    env[0] = 0;
  }

  i = dollar - str;
  strncpy (buf, str, i);
  buf[i] = 0;
  strncat (buf, e, sizeof(buf)-1-i);
  return strcat (buf, p);
}

/*
 * Set value for right-side (name) to value. Only parse Watt-32
 * keyword needed by tcptrace.
 */
static void set_values (char *name, char *value)
{
  char val[80];

  strncpy (val, value, sizeof(val)-1);
  val [sizeof(val)-1] = 0;
  strupr (name);

  value = ExpandVarStr (value);

  if (!strcmp(name,"MY_IP"))
  {
    if (debug > 2)
       fprintf (stderr, "Setting ip-addr `%s'\n", value);
    my_ip_addr = aton (value);
  }
  else if (!strcmp(name,"HOSTNAME"))
  {
    if (debug > 2)
       fprintf (stderr, "Setting hostname `%s'\n", value);
    strncpy (_hostname, value, sizeof(_hostname)-1);
    _hostname [sizeof(_hostname)-1] = 0;
  }
  else if (!strcmp(name,"DOMAINSLIST") || !strcmp(name,"DOMAIN_LIST"))
  {
    if (debug > 2)
       fprintf (stderr, "Setting domain `%s'\n", value);
    setdomainname (value, sizeof(defaultdomain)-1);
  }
  else if (!strcmp(name,"HOSTS"))
  {
    if (debug > 2)
       fprintf (stderr, "Opening hosts file `%s'\n", value);
    ReadHostFile (value);
  }
  else if (!strcmp(name,"SERVICES"))
  {
    if (debug > 2)
       fprintf (stderr, "Opening service file `%s'\n", value);
    ReadServFile (value);
  }
}


int tcp_config (const char *path)
{
  char  name [PATH_MAX];
  char  value[80], ch[2];
  int   quotemode, f, mode;

  strcpy (name, "$(WATTCP.CFG)");
  strcat (name, "/WATTCP.CFG");

  path = ExpandVarStr (name);
  if (path == name)
  {
    fprintf (stderr, "`$(WATTCP.CFG)/wattcp.cfg' not found\n");
    return (0);
  }

  if (debug > 2)
     fprintf (stderr, "Opening `$(WATTCP.CFG)/wattcp.cfg' = `%s'\n", path);

  f = open (path, O_RDONLY|O_TEXT);
  if (f < 0)
  {
    fprintf (stderr, "Cannot open `%s'\n", path);
    return (0);
  }

  /* clear defaults
   */
  memset (&_hostname, 0, sizeof(_hostname));
  memset (&defaultdomain,0, sizeof(defaultdomain));

  *name = *value = mode = ch[1] = 0;
  quotemode = 0;

  while (read(f,(char*)&ch,1) == 1)
  {
    switch (ch[0])
    {
      case '\"': quotemode ^= 1;
                 break;
      case ' ' :
      case '\t': if (quotemode)
                    goto addit;
                 break;

      case '=' : if (quotemode)
                    goto addit;
                 if (!mode)
                    mode = 1;
                 break;
      case '#' :
      case ';' : if (quotemode)
                    goto addit;
                 mode = 2;   /* comment */
                 break;

      case '\n':
      case '\r': 
                 if (*name && *value)
                     set_values (name,value);
                 *name = *value = quotemode = 0;
                 mode = 0;
                 break;
      default:
    addit:       switch (mode)
                 {
                   case 0 : strcat (name,ch);
                            break;
                   case 1 : strcat (value,ch);
                            break;
                 }
                 break;
    }
  }
  close (f);
  return (1);
}

/*
 * return TRUE is ip address is local. ip is on host order.
 */
int is_local_addr (DWORD ip)
{
  return ((DWORD)(ip - my_ip_addr) <= multihomes || (ip >> 24) == 127);
    /* (my_ip_addr <= ip <= my_ip_addr+multihomes) or loopback address */
}

/*
 * Return TRUE if ip address is a legal address for
 * a unique host. `ip' is on host order.
 */
int is_unique_addr (DWORD ip)
{
  return (ip && (ip & ~sin_mask) == 0);
}

/*
 * return TRUE if ip packet is a (limited) broadcast packet.
 */
int is_ip_brdcast (struct ip *ip)
{
  DWORD dst = intel (ip->ip_dst.s_addr);
  return ((~dst & ~sin_mask) == 0); /* (limited) ip-broadcast */
}

/*
 * return TRUE if IP addr is Class D (Multicast)
 */
int is_multicast (DWORD ip)
{
  if (IN_CLASSD(ip))
     return (1);
  return (0);
}
